# Generated by roxygen2: do not edit by hand

S3method(ccdf,prior)
S3method(cdf,prior)
S3method(density,prior)
S3method(lines,prior)
S3method(lpdf,prior)
S3method(mccdf,prior)
S3method(mcdf,prior)
S3method(mean,prior)
S3method(mlpdf,prior)
S3method(mpdf,prior)
S3method(mquant,prior)
S3method(pdf,default)
S3method(pdf,prior)
S3method(plot,prior)
S3method(print,BayesTools_table)
S3method(print,prior)
S3method(quant,prior)
S3method(range,prior)
S3method(rng,prior)
S3method("[",BayesTools_BF)
S3method("[",BayesTools_table)
S3method(sd,default)
S3method(sd,prior)
S3method(update,BayesTools_table)
S3method(var,default)
S3method(var,prior)
export(JAGS_add_priors)
export(JAGS_bridgesampling)
export(JAGS_bridgesampling_posterior)
export(JAGS_check_and_list_autofit_settings)
export(JAGS_check_and_list_fit_settings)
export(JAGS_check_convergence)
export(JAGS_diagnostics)
export(JAGS_diagnostics_autocorrelation)
export(JAGS_diagnostics_density)
export(JAGS_diagnostics_trace)
export(JAGS_estimates_empty_table)
export(JAGS_estimates_table)
export(JAGS_evaluate_formula)
export(JAGS_extend)
export(JAGS_fit)
export(JAGS_formula)
export(JAGS_formula_design)
export(JAGS_get_inits)
export(JAGS_inference_empty_table)
export(JAGS_inference_table)
export(JAGS_marglik_parameters)
export(JAGS_marglik_parameters_formula)
export(JAGS_marglik_priors)
export(JAGS_marglik_priors_formula)
export(JAGS_parameter_names)
export(JAGS_summary_table)
export(JAGS_to_monitor)
export(Savage_Dickey_BF)
export(add_column)
export(as_marginal_inference)
export(as_mixed_posteriors)
export(bridgesampling_object)
export(ccdf)
export(cdf)
export(check_bool)
export(check_char)
export(check_int)
export(check_list)
export(check_real)
export(compute_inference)
export(contr.independent)
export(contr.meandif)
export(contr.orthonormal)
export(dmpoint)
export(dpoint)
export(ensemble_diagnostics_empty_table)
export(ensemble_diagnostics_table)
export(ensemble_estimates_empty_table)
export(ensemble_estimates_table)
export(ensemble_inference)
export(ensemble_inference_empty_table)
export(ensemble_inference_table)
export(ensemble_summary_empty_table)
export(ensemble_summary_table)
export(format_BF)
export(format_parameter_names)
export(formula_add_intercept)
export(geom_prior)
export(geom_prior_list)
export(inclusion_BF)
export(interpret)
export(interpret2)
export(interpret_records)
export(interpret_tables)
export(is.prior)
export(is.prior.PEESE)
export(is.prior.PET)
export(is.prior.discrete)
export(is.prior.factor)
export(is.prior.independent)
export(is.prior.meandif)
export(is.prior.mixture)
export(is.prior.none)
export(is.prior.orthonormal)
export(is.prior.point)
export(is.prior.simple)
export(is.prior.spike_and_slab)
export(is.prior.treatment)
export(is.prior.vector)
export(is.prior.weightfunction)
export(is_prior_bias)
export(is_prior_phacking)
export(lines_prior_list)
export(lpdf)
export(marginal_estimates_table)
export(marginal_inference)
export(marginal_posterior)
export(mccdf)
export(mcdf)
export(mdone.sided)
export(mdone.sided_fixed)
export(mdtwo.sided)
export(mdtwo.sided_fixed)
export(mix_posteriors)
export(mlpdf)
export(model_summary_empty_table)
export(model_summary_table)
export(models_inference)
export(mpdf)
export(mpone.sided)
export(mpone.sided_fixed)
export(mptwo.sided)
export(mptwo.sided_fixed)
export(mqone.sided)
export(mqone.sided_fixed)
export(mqtwo.sided)
export(mqtwo.sided_fixed)
export(mquant)
export(pdf)
export(phack_alpha_from_pi_null)
export(phack_backend_constants)
export(phack_pi_null)
export(plot_marginal)
export(plot_models)
export(plot_posterior)
export(plot_prior_list)
export(plot_transformed_prior)
export(pmpoint)
export(ppoint)
export(prior)
export(prior_PEESE)
export(prior_PET)
export(prior_bias)
export(prior_factor)
export(prior_informed)
export(prior_informed_medicine_names)
export(prior_mixture)
export(prior_none)
export(prior_phacking)
export(prior_spike_and_slab)
export(prior_weightfunction)
export(qmpoint)
export(qpoint)
export(quant)
export(remove_column)
export(rmpoint)
export(rng)
export(rone.sided)
export(rone.sided_fixed)
export(rpoint)
export(rtwo.sided)
export(rtwo.sided_fixed)
export(runjags_estimates_empty_table)
export(runjags_estimates_table)
export(runjags_inference_empty_table)
export(runjags_inference_table)
export(sd)
export(selection_backend_spec)
export(stan_estimates_table)
export(transform_factor_samples)
export(transform_meandif_samples)
export(transform_orthonormal_samples)
export(transform_prior_samples)
export(transform_scale_samples)
export(var)
export(weightfunctions_mapping)
export(wf_cumulative)
export(wf_fixed)
export(wf_independent)
importFrom(Rdpack,reprompt)
importFrom(grDevices,pdf)
importFrom(rlang,.data)
importFrom(stats,density)
importFrom(stats,fft)
importFrom(stats,median)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,var)
importFrom(utils,head)
importFrom(utils,tail)
