##' BayesTools
##'
##' BayesTools: Provides tools for conducting Bayesian analyses. The package contains
##' functions for creating a wide range of prior distribution objects, mixing posterior
##' samples from JAGS and Stan models, plotting posterior distributions, and etc...
##' The tools for working with prior distribution span from visualization, generating JAGS
##' and bridgesampling syntax to basic functions such as rng, quantile, and distribution functions.
##'
##'
##' @name BayesTools
##' @aliases BayesTools-package BayesTools
##' @docType package
##' @author František Bartoš \email{f.bartos96@@gmail.com}
##' @keywords package
##' @importFrom rlang .data
##' @importFrom Rdpack reprompt
##' @importFrom stats fft median setNames
##' @importFrom utils head tail
"_PACKAGE"
