#' @title Kitchen Rolls data from
#' \insertCite{wagenmakers2015turning;textual}{BayesTools} replication study.
#'
#' @description The data set contains mean NEO PI-R scores for two groups of students.
#' Each of them filled a personality questionnaire while rotating a kitchen roll either
#' clock or counter-clock wise. See \insertCite{wagenmakers2015turning;textual}{BayesTools}
#' for more details about the replication study and the https://osf.io/uszvx/ for the
#' original data.
#'
#' @format A data.frame with 2 columns and 102 observations.
#'
#' @return a data.frame.
#'
#' @references
#' \insertAllCited{}
"kitchen_rolls"
