% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BayesTools-package.R
\docType{package}
\name{BayesTools}
\alias{BayesTools}
\alias{BayesTools-package}
\title{BayesTools}
\description{
BayesTools: Provides tools for conducting Bayesian analyses. The package contains
functions for creating a wide range of prior distribution objects, mixing posterior
samples from JAGS and Stan models, plotting posterior distributions, and etc...
The tools for working with prior distribution span from visualization, generating JAGS
and bridgesampling syntax to basic functions such as rng, quantile, and distribution functions.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://fbartos.github.io/BayesTools/}
  \item Report bugs at \url{https://github.com/FBartos/BayesTools/issues}
}

}
\author{
František Bartoš \email{f.bartos96@gmail.com}
}
\keyword{package}
