% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/JAGS-fit.R
\name{JAGS_add_priors}
\alias{JAGS_add_priors}
\title{Add 'JAGS' prior}
\usage{
JAGS_add_priors(syntax, prior_list)
}
\arguments{
\item{syntax}{JAGS model syntax}

\item{prior_list}{named list of prior distribution
(names correspond to the parameter names)}
}
\value{
\code{JAGS_add_priors} returns a JAGS syntax.
}
\description{
Adds priors to a 'JAGS' syntax.
}
