% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary-tables.R
\name{remove_column}
\alias{remove_column}
\title{Removes column to BayesTools table}
\usage{
remove_column(table, column_position = NULL)
}
\arguments{
\item{table}{BayesTools table}

\item{column_position}{position of the to be removed column (defaults to \code{NULL} which
removes the last column)}
}
\value{
returns an object of 'BayesTools_table' class.
}
\description{
Removes column to a BayesTools table while not
breaking formatting, attributes, etc...
}
