% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary-tables.R
\name{print.BayesTools_table}
\alias{print.BayesTools_table}
\title{Print a BayesTools table}
\usage{
\method{print}{BayesTools_table}(x, ...)
}
\arguments{
\item{x}{a BayesTools_values_tables}

\item{...}{additional arguments.}
}
\value{
\code{print.BayesTools_table} returns \code{NULL}.
}
\description{
Print a BayesTools table
}
