% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/JAGS-marglik.R
\name{bridgesampling_object}
\alias{bridgesampling_object}
\title{Create a 'bridgesampling' object}
\usage{
bridgesampling_object(logml = -Inf)
}
\arguments{
\item{logml}{log marginal likelihood. Defaults to \code{-Inf}.}
}
\value{
\code{JAGS_bridgesampling} returns an object of class 'bridge'.
}
\description{
prepares a 'bridgesampling' object with a given
log marginal likelihood.
}
