% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/priors.R
\name{var}
\alias{var}
\title{Creates generic for var function}
\usage{
var(x, ...)
}
\arguments{
\item{x}{main argument}

\item{...}{additional arguments}
}
\value{
\code{var} returns a variance
of the supplied object (if it is either a numeric vector
or an object of class 'prior').
}
\description{
Creates generic for var function
}
\seealso{
\link[stats]{cor}
}
