% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/selection-kernels.R
\name{prior_bias}
\alias{prior_bias}
\title{Creates a composed publication-bias prior}
\usage{
prior_bias(selection = NULL, phacking = NULL, prior_weights = 1)
}
\arguments{
\item{selection}{optional \code{prior_weightfunction()} object.}

\item{phacking}{optional \code{prior_phacking()} object.}

\item{prior_weights}{prior odds associated with a given distribution.}
}
\value{
\code{prior_bias()} returns an object of class \code{"prior"}.
}
\description{
\code{prior_bias()} composes a step-selection
\code{prior_weightfunction()} and/or a \code{prior_phacking()} object. It does
not introduce new math; the backend compiler treats the composition as the
product of both kernels.
}
