% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/model-averaging-plots.R
\name{plot_marginal}
\alias{plot_marginal}
\title{Plot samples from the marginal posterior distributions}
\usage{
plot_marginal(
  samples,
  parameter,
  plot_type = "base",
  prior = FALSE,
  n_points = 1000,
  transformation = NULL,
  transformation_arguments = NULL,
  transformation_settings = FALSE,
  rescale_x = FALSE,
  par_name = NULL,
  dots_prior = list(),
  legend = TRUE,
  legend_title = NULL,
  legend_labels = NULL,
  legend_position = NULL,
  ...
)
}
\arguments{
\item{samples}{samples from a posterior distribution for a
parameter generated by \link{marginal_inference}.}

\item{parameter}{parameter name to be plotted.}

\item{plot_type}{whether to use a base plot \code{"base"}
or ggplot2 \code{"ggplot"} for plotting.}

\item{prior}{whether prior distribution should be added to the figure.
When samples were prepared with \code{as_mixed_posteriors(..., transform_scaled = TRUE)},
the transformed prior samples are automatically used.}

\item{n_points}{number of equally spaced points
in the \code{x_range} if \code{x_seq} is unspecified}

\item{transformation}{transformation to be applied
to the prior distribution. Either a character
specifying one of the prepared transformations:
\describe{
\item{lin}{linear transformation in form of \code{a + b*x}}
\item{tanh}{hyperbolic tangent transformation}
\item{exp}{exponential transformation}
}, or a list containing the transformation function \code{fun},
inverse transformation function \code{inv}, and derivative of the
transformation \code{jac}, evaluated on the original support. See examples
for details.}

\item{transformation_arguments}{a list with named arguments for
the \code{transformation}}

\item{transformation_settings}{boolean indicating whether the
settings the \code{x_seq} or \code{x_range} was specified on
the transformed support}

\item{rescale_x}{allows to rescale x-axis in case a
weightfunction is plotted.}

\item{par_name}{a type of parameter for which the prior is
specified. Only relevant if the prior corresponds to a mu
parameter that needs to be transformed.}

\item{dots_prior}{additional arguments for the prior distribution plot}

\item{legend}{whether factor legends should be drawn.}

\item{legend_title}{optional title for factor legends.}

\item{legend_labels}{optional labels for factor legend levels.}

\item{legend_position}{optional legend position for factor legends.}

\item{...}{additional arguments}
}
\value{
\code{plot_marginal} returns either \code{NULL} or
an object of class 'ggplot' if plot_type is \code{plot_type = "ggplot"}.
}
\description{
Plot samples from the marginal posterior distributions
}
\seealso{
\code{\link[=prior]{prior()}} \code{\link[=marginal_inference]{marginal_inference()}}  \code{\link[=plot_posterior]{plot_posterior()}}
}
