% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/priors.R
\name{sd}
\alias{sd}
\title{Creates generic for sd function}
\usage{
sd(x, ...)
}
\arguments{
\item{x}{main argument}

\item{...}{additional arguments}
}
\value{
\code{sd} returns a standard deviation
of the supplied object (if it is either a numeric vector
or an object of class 'prior').
}
\description{
Creates generic for sd function
}
\seealso{
\link[stats]{sd}
}
