% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/priors-density.R
\name{range.prior}
\alias{range.prior}
\title{Prior range}
\usage{
\method{range}{prior}(x, quantiles = NULL, ..., na.rm = FALSE)
}
\arguments{
\item{x}{a prior}

\item{quantiles}{quantile to be returned in
case of unbounded distribution.}

\item{...}{additional arguments}

\item{na.rm}{unused}
}
\value{
\code{range.prior} returns a numeric vector of
length with a plotting range of a prior distribution.
}
\description{
Computes range of a prior
distribution (if the prior distribution is
unbounded range from \code{quantiles} to
\code{1 -quantiles}) is returned.
}
\seealso{
\code{\link[=prior]{prior()}}
}
