% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary-tables.R
\name{add_column}
\alias{add_column}
\title{Adds column to BayesTools table}
\usage{
add_column(
  table,
  column_title,
  column_values,
  column_position = NULL,
  column_type = NULL
)
}
\arguments{
\item{table}{BayesTools table}

\item{column_title}{title of the new column}

\item{column_values}{values of the new column}

\item{column_position}{position of the new column (defaults to \code{NULL} which
appends the column to the end)}

\item{column_type}{type of values of the new column table (important for formatting,
defaults to \code{NULL} = the function tries to guess numeric / character based on the
\code{column_values} but many more specific types are available)}
}
\value{
returns an object of 'BayesTools_table' class.
}
\description{
Adds column to a BayesTools table while not
breaking formatting, attributes, etc...
}
