% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/selection-kernels.R
\name{phack_pi_null}
\alias{phack_pi_null}
\alias{phack_alpha_from_pi_null}
\alias{phack_backend_constants}
\title{P-hacking calibration helpers}
\usage{
phack_pi_null(alpha, form, source, destination, target = 0.025)

phack_alpha_from_pi_null(pi_null, form, source, destination, target = 0.025)

phack_backend_constants(form, source, destination, target = 0.025)
}
\arguments{
\item{alpha}{p-hacking severity parameter.}

\item{form}{power depletion form, either \code{"linear"} or
\code{"quadratic"}.}

\item{source}{source p-value cut point.}

\item{destination}{destination p-value cut point.}

\item{target}{target p-value cut point.}

\item{pi_null}{null probability mass depleted from the source interval.}
}
\value{
Numeric vector for the calibration helpers and a named list for
\code{phack_backend_constants()}.
}
\description{
\code{phack_pi_null()} converts the sampled severity
\code{alpha} to the amount of null probability mass depleted from the source
interval. \code{phack_alpha_from_pi_null()} applies the inverse calibration.
\code{phack_backend_constants()} returns the z-scale constants used by the
backend.
}
