% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/JAGS-marglik.R
\name{JAGS_marglik_priors}
\alias{JAGS_marglik_priors}
\alias{JAGS_marglik_priors_formula}
\title{Compute marginal likelihood for 'JAGS' priors}
\usage{
JAGS_marglik_priors(samples, prior_list)

JAGS_marglik_priors_formula(samples, formula_prior_list)
}
\arguments{
\item{samples}{samples provided by bridgesampling
function}

\item{prior_list}{named list of prior distribution
(names correspond to the parameter names) of parameters not specified within the
\code{formula_list}}

\item{formula_prior_list}{named list of named lists of prior distributions
(names of the lists correspond to the parameter name created by each of the formula and
the names of the prior distribution correspond to the parameter names) of parameters specified
within the \code{formula}}
}
\value{
\code{JAGS_marglik_priors} returns a numeric value
of likelihood evaluated at the current posterior sample.
}
\description{
Computes marginal likelihood for the
prior part of a 'JAGS' model within 'bridgesampling'
function
}
