% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary-tables.R
\name{update.BayesTools_table}
\alias{update.BayesTools_table}
\title{Updates BayesTools table}
\usage{
\method{update}{BayesTools_table}(
  object,
  title = NULL,
  footnotes = NULL,
  warnings = NULL,
  remove_parameters = NULL,
  logBF = FALSE,
  BF01 = FALSE,
  ...
)
}
\arguments{
\item{object}{a BayesTools table}

\item{title}{title of the table}

\item{footnotes}{add footnotes to the table}

\item{warnings}{add warnings of the table}

\item{remove_parameters}{remove parameters from the table}

\item{logBF}{whether to format Bayes factors as log(BF)}

\item{BF01}{whether to format Bayes factors as 1/BF}

\item{...}{additional arguments.}
}
\value{
returns an object of 'BayesTools_table' class.
}
\description{
Updates BayesTools table while not breaking formatting, attributes, etc...
}
