% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/model-averaging.R
\name{weightfunctions_mapping}
\alias{weightfunctions_mapping}
\title{Create coefficient mapping between multiple weightfunctions}
\usage{
weightfunctions_mapping(prior_list, cuts_only = FALSE, one_sided = FALSE)
}
\arguments{
\item{prior_list}{list of prior distributions}

\item{cuts_only}{whether only p-value cuts should be returned}

\item{one_sided}{force one-sided output}
}
\value{
\code{weightfunctions_mapping} returns a list of indices
mapping the publication weights omega from the individual weightfunctions
into a joint weightfunction.
}
\description{
Creates coefficients mapping between multiple weightfunctions.
}
