% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/JAGS-formula.R
\name{formula_add_intercept}
\alias{formula_add_intercept}
\title{Add an Intercept to a Formula}
\usage{
formula_add_intercept(formula)
}
\arguments{
\item{formula}{a formula object.}
}
\value{
A formula object with an intercept.
}
\description{
Converts a no-intercept formula to the corresponding formula
with an intercept while preserving the formula environment. Top-level
no-intercept encodings such as \code{- 1}, \code{+ 0}, and \code{0 +} are
removed without editing transformed calls such as \code{I(x - 1)} or
\code{offset(x - 1)}.
}
