% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/JAGS-formula.R
\name{transform_orthonormal_samples}
\alias{transform_orthonormal_samples}
\title{Transform orthonomal posterior samples into differences from the mean}
\usage{
transform_orthonormal_samples(samples)
}
\arguments{
\item{samples}{(a list) of mixed posterior distributions created with
\code{mix_posteriors} function}
}
\value{
\code{transform_orthonormal_samples} returns a named list of mixed posterior
distributions (either a vector of matrix).
}
\description{
Transforms posterior samples from model-averaged posterior
distributions based on orthonormal prior distributions into differences from
the mean.
}
\seealso{
\link{mix_posteriors} \link{contr.orthonormal}
}
