% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/priors.R
\name{var.prior}
\alias{var.prior}
\title{Prior var}
\usage{
\method{var}{prior}(x, ...)
}
\arguments{
\item{x}{a prior}

\item{...}{unused arguments}
}
\value{
a variance of an object of class 'prior'.
}
\description{
Computes variance
of a prior distribution.
}
\examples{
# create a standard normal prior distribution
p1 <- prior(distribution = "normal", parameters = list(mean = 1, sd = 1))

# compute variance of the prior distribution
var(p1)

}
\seealso{
\code{\link[=prior]{prior()}}
}
