% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{kitchen_rolls}
\alias{kitchen_rolls}
\title{Kitchen Rolls data from
\insertCite{wagenmakers2015turning;textual}{BayesTools} replication study.}
\format{
A data.frame with 2 columns and 102 observations.
}
\usage{
kitchen_rolls
}
\value{
a data.frame.
}
\description{
The data set contains mean NEO PI-R scores for two groups of students.
Each of them filled a personality questionnaire while rotating a kitchen roll either
clock or counter-clock wise. See \insertCite{wagenmakers2015turning;textual}{BayesTools}
for more details about the replication study and the https://osf.io/uszvx/ for the
original data.
}
\references{
\insertAllCited{}
}
\keyword{datasets}
