% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/selection-kernels.R
\name{is_prior_bias}
\alias{is_prior_bias}
\title{Reports whether x is a composed publication-bias prior}
\usage{
is_prior_bias(x)
}
\arguments{
\item{x}{object to test.}
}
\value{
A logical value.
}
\description{
Reports whether x is a composed publication-bias prior
}
