% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary-tables.R
\name{format_BF}
\alias{format_BF}
\title{Format Bayes factor}
\usage{
format_BF(BF, logBF = FALSE, BF01 = FALSE, inclusion = FALSE)
}
\arguments{
\item{BF}{Bayes factor(s)}

\item{logBF}{log(BF)}

\item{BF01}{1/BF}

\item{inclusion}{whether the Bayes factor is an inclusion BF (for naming purposes)}
}
\value{
\code{format_BF} returns a formatted Bayes factor.
}
\description{
Formats Bayes factor
}
