% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/JAGS-marglik.R
\name{JAGS_marglik_parameters}
\alias{JAGS_marglik_parameters}
\alias{JAGS_marglik_parameters_formula}
\title{Extract parameters for 'JAGS' priors}
\usage{
JAGS_marglik_parameters(samples, prior_list)

JAGS_marglik_parameters_formula(
  samples,
  formula_list,
  formula_data_list,
  formula_prior_list,
  prior_list_parameters
)
}
\arguments{
\item{samples}{samples provided by bridgesampling
function}

\item{prior_list}{named list of prior distribution
(names correspond to the parameter names) of parameters not specified within the
\code{formula_list}}

\item{formula_list}{named list of formulas to be added to the model
(names correspond to the parameter name created by each of the formula)}

\item{formula_data_list}{named list of data frames containing data for each formula
(names of the lists correspond to the parameter name created by each of the formula)}

\item{formula_prior_list}{named list of named lists of prior distributions
(names of the lists correspond to the parameter name created by each of the formula and
the names of the prior distribution correspond to the parameter names) of parameters specified
within the \code{formula}}

\item{prior_list_parameters}{named list of prior distributions on model parameters
(not specified within the formula but that might scale the formula parameters)}
}
\value{
\code{JAGS_marglik_parameters} returns a named list
of (transformed) posterior samples.
}
\description{
Extracts transformed parameters from the
prior part of a 'JAGS' model inside of a 'bridgesampling'
function (returns them as a named list)
}
