% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/selection-kernels.R
\name{prior_phacking}
\alias{prior_phacking}
\title{Creates a prior distribution for a p-hacking selection kernel}
\usage{
prior_phacking(
  side = "one-sided",
  target = 0.025,
  source = 0.25,
  destination = 0.005,
  form = c("linear", "quadratic"),
  alpha = prior("beta", list(1, 1)),
  report_scale = "pi_null",
  prior_weights = 1
)
}
\arguments{
\item{side}{side geometry. Currently only \code{"one-sided"} is supported.}

\item{target}{target p-value cut point.}

\item{source}{source p-value cut point. Must be larger than \code{target}.}

\item{destination}{destination p-value cut point. Must be smaller than
\code{target}.}

\item{form}{power depletion form, either \code{"linear"} or
\code{"quadratic"}.}

\item{alpha}{prior distribution for the p-hacking severity parameter.}

\item{report_scale}{reporting scale for deterministic summaries.}

\item{prior_weights}{prior odds associated with a given distribution.}
}
\value{
\code{prior_phacking()} returns an object of class \code{"prior"}.
}
\description{
\code{prior_phacking()} creates a prior distribution for a
mass-preserving p-hacking redistribution kernel. The initial backend supports
linear and quadratic power depletion between \code{source} and
\code{target}, with depleted probability mass redistributed between
\code{target} and \code{destination}.
}
