% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/JAGS-formula.R
\name{JAGS_evaluate_formula}
\alias{JAGS_evaluate_formula}
\title{Evaluate JAGS formula using posterior samples}
\usage{
JAGS_evaluate_formula(fit, formula, parameter, data, prior_list)
}
\arguments{
\item{fit}{model fitted with either \link[runjags]{runjags} posterior
samples obtained with \link[rjags]{rjags-package}}

\item{formula}{formula specifying the right hand side of the assignment (the
left hand side is ignored). If the formula has a \code{"log(intercept)"}
attribute set to \code{TRUE}, the intercept values will be log-transformed
before computing the linear predictor.}

\item{parameter}{name of the parameter created with the formula}

\item{data}{data.frame containing predictors included in the formula}

\item{prior_list}{named list of prior distribution of parameters specified within
the \code{formula}}
}
\value{
\code{JAGS_evaluate_formula} returns a matrix of the evaluated posterior samples on
the supplied data.
}
\description{
Evaluates a JAGS formula on a posterior distribution
obtained from a fitted model.
}
\seealso{
\code{\link[=JAGS_fit]{JAGS_fit()}} \code{\link[=JAGS_formula]{JAGS_formula()}}
}
